@@ -5,7 +5,7 @@
 
     File utility functions for Sphinx.
 
-    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2017 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 from __future__ import absolute_import
@@ -16,8 +16,15 @@ import posixpath
 from docutils.utils import relative_path
 from sphinx.util.osutil import copyfile, ensuredir, walk
 
+if False:
+    # For type annotation
+    from typing import Callable, Dict, Union  # NOQA
+    from sphinx.util.matching import Matcher  # NOQA
+    from sphinx.util.template import BaseRenderer  # NOQA
+
 
 def copy_asset_file(source, destination, context=None, renderer=None):
+    # type: (unicode, unicode, Dict, BaseRenderer) -> None
     """Copy an asset file to destination.
 
     On copying, it expands the template variables if context argument is given and
@@ -40,16 +47,17 @@ def copy_asset_file(source, destination, context=None, renderer=None):
             from sphinx.util.template import SphinxRenderer
             renderer = SphinxRenderer()
 
-        with codecs.open(source, 'r', encoding='utf-8') as fsrc:
+        with codecs.open(source, 'r', encoding='utf-8') as fsrc:  # type: ignore
             if destination.lower().endswith('_t'):
                 destination = destination[:-2]
-            with codecs.open(destination, 'w', encoding='utf-8') as fdst:
+            with codecs.open(destination, 'w', encoding='utf-8') as fdst:  # type: ignore
                 fdst.write(renderer.render_string(fsrc.read(), context))
     else:
         copyfile(source, destination)
 
 
 def copy_asset(source, destination, excluded=lambda path: False, context=None, renderer=None):
+    # type: (unicode, unicode, Union[Callable[[unicode], bool], Matcher], Dict, BaseRenderer) -> None  # NOQA
     """Copy asset files to destination recursively.
 
     On copying, it expands the template variables if context argument is given and
